# This file is part of BurnMan - a thermoelastic and thermodynamic toolkit for
# the Earth and Planetary Sciences
# Copyright (C) 2012 - 2024 by the BurnMan team, released under the GNU
# GPL v2 or later.

from burnman.eos.birch_murnaghan import BirchMurnaghanBase as BM3
from burnman.eos.mie_grueneisen_debye import MGDBase
from burnman.classes.calibrant import Calibrant


"""
Farla_2025
^^^^^^^^^^^
"""


class RbCl(Calibrant):
    """
    The RbCl pressure standard reported by
    Farla et al. (2025; DOI ).
    """

    def __init__(self):
        def _pressure_Farla_RbCl(volume, temperature, params):

            # Isothermal pressure (GPa)
            pressure_model = BM3()
            P0 = pressure_model.pressure(params["T_0"], volume, params)

            # Thermal pressure
            thermal_model = MGDBase()
            Pth0 = thermal_model._thermal_pressure(params["T_0"], volume, params)
            Pth = thermal_model._thermal_pressure(temperature, volume, params)

            # Total pressure
            P = P0 + Pth - Pth0

            return P

        _params_Farla_RbCl = {
            "V_0": 3.6240E-05,
            "K_0": 20.46743e9,
            "Kprime_0": 4.93095,
            "Debye_0": 177.04,
            "grueneisen_0": 1.99761,
            "q_0": 1.39675,
            "n": 2.0,
            "T_0": 300.0,
            "P_0": 0.0,
            "Z": 1.0,
        }

        Calibrant.__init__(self, _pressure_Farla_RbCl, "pressure", _params_Farla_RbCl)

class RbBr(Calibrant):
    """
    The RbBr pressure standard reported by
    Farla et al. (2025; DOI ).
    """

    def __init__(self):
        def _pressure_Farla_RbBr(volume, temperature, params):

            # Isothermal pressure (GPa)
            pressure_model = BM3()
            P0 = pressure_model.pressure(params["T_0"], volume, params)

            # Thermal pressure
            thermal_model = MGDBase()
            Pth0 = thermal_model._thermal_pressure(params["T_0"], volume, params)
            Pth = thermal_model._thermal_pressure(temperature, volume, params)

            # Total pressure
            P = P0 + Pth - Pth0

            return P

        _params_Farla_RbBr = {
            "V_0": 4.170E-05,
            "K_0": 17.04423e9,
            "Kprime_0": 5.12548,
            "Debye_0": 144.96,
            "grueneisen_0": 2.21528,
            "q_0": 1.89904,
            "n": 2.0,
            "T_0": 300.0,
            "P_0": 0.0,
            "Z": 1.0,
        }

        Calibrant.__init__(self, _pressure_Farla_RbBr, "pressure", _params_Farla_RbBr)
        
class RbI(Calibrant):
    """
    The RbI pressure standard reported by
    Farla et al. (2025; DOI ).
    """

    def __init__(self):
        def _pressure_Farla_RbI(volume, temperature, params):

            # Isothermal pressure (GPa)
            pressure_model = BM3()
            P0 = pressure_model.pressure(params["T_0"], volume, params)

            # Thermal pressure
            thermal_model = MGDBase()
            Pth0 = thermal_model._thermal_pressure(params["T_0"], volume, params)
            Pth = thermal_model._thermal_pressure(temperature, volume, params)

            # Total pressure
            P = P0 + Pth - Pth0

            return P

        _params_Farla_RbI = {
            "V_0": 5.1090E-05,
            "K_0": 14.17143e9,
            "Kprime_0": 4.86818,
            "Debye_0": 117.98,
            "grueneisen_0": 2.27564,
            "q_0": 1.84641,
            "n": 2.0,
            "T_0": 300.0,
            "P_0": 0.0,
            "Z": 1.0,
        }

        Calibrant.__init__(self, _pressure_Farla_RbI, "pressure", _params_Farla_RbI)        