# This file is part of EosCross, a utility to jointly estimate pressure and temperature, written by R. Farla.

import numpy as np
import os
from burnman.calibrants.Zeng_2010 import Mo
from tkinter import messagebox

def read_material_data(file_path):
    material_1 = {'temperature': [], 'unit_cell_param': [], 'unit_cell_param_error': [], 
                  'unit_cell_volume': [], 'unit_cell_volume_error': [], 'density': []}
    material_2 = {'temperature': [], 'unit_cell_param': [], 'unit_cell_param_error': [], 
                  'unit_cell_volume': [], 'unit_cell_volume_error': [], 'density': []}
    
    with open(file_path, 'r') as file:
        for line in file:
            values = line.split()
            if len(values) == 6:  # Only material 1 data
                temp, uc_param, uc_param_err, uc_volume, uc_volume_err, density = map(float, values)
                material_1['temperature'].append(temp)
                material_1['unit_cell_param'].append(uc_param)
                material_1['unit_cell_param_error'].append(uc_param_err)
                material_1['unit_cell_volume'].append(uc_volume)
                material_1['unit_cell_volume_error'].append(uc_volume_err)
                material_1['density'].append(density)
            elif len(values) == 11:  # Material 1 and Material 2 data
                temp, uc_param1, uc_param_err1, uc_volume1, uc_volume_err1, density1, \
                uc_param2, uc_param_err2, uc_volume2, uc_volume_err2, density2 = map(float, values)
                material_1['temperature'].append(temp)
                material_1['unit_cell_param'].append(uc_param1)
                material_1['unit_cell_param_error'].append(uc_param_err1)
                material_1['unit_cell_volume'].append(uc_volume1)
                material_1['unit_cell_volume_error'].append(uc_volume_err1)
                material_1['density'].append(density1)
                material_2['temperature'].append(temp)
                material_2['unit_cell_param'].append(uc_param2)
                material_2['unit_cell_param_error'].append(uc_param_err2)
                material_2['unit_cell_volume'].append(uc_volume2)
                material_2['unit_cell_volume_error'].append(uc_volume_err2)
                material_2['density'].append(density2)
    
    return material_1, material_2

def open_file(a0, Tesd):
    with open('path.txt', 'r') as file:
        file_path = file.readline().strip()
    
    if file_path:
        material_1, material_2 = read_material_data(file_path)
        T, P2, P_esd2 = EoS2recalc(material_2, a0, Tesd)
        
        if T and P2 and P_esd2:  # Ensure data exists before saving
            save_pressures_to_file(T, P2, P_esd2, 'Pressures_Mo_Zeng2010.txt')
            messagebox.showinfo("Success", f"Data imported, pressures calculated, and saved in Pressures_Mo_Zeng2010.txt.\nThere were {len(material_1['temperature'])} entries.")
        else:
            messagebox.showwarning("Warning", "No data processed. Check input file or calculations.")

def EoS2recalc(material_2, a0, Tesd):
    P2 = []
    P_esd2 = []
    T = []

    phase = Mo()
    Na = 6.0221367e+23
    
    for i in range(len(material_2['temperature'])):
        unit_cell_param = material_2['unit_cell_param'][i]
        unit_cell_param_error = material_2['unit_cell_param_error'][i]
        temperature_phase = material_2['temperature'][i]
        
        # Append temperature to T
        T.append(temperature_phase)
        
        volume = (Na / phase.params['Z']) * (unit_cell_param ** 3) * (10 ** -30)
        phase.params['V_0'] = (Na / phase.params['Z']) * (float(a0.get()) ** 3) * (10 ** -30)
        
        s_volume = 3 * (unit_cell_param ** 2) * unit_cell_param_error
        sigma_volume = (Na / phase.params['Z']) * (s_volume) * (10 ** -30)
        
        pressure = phase.pressure_function(volume, temperature_phase, phase.params) * 1.0e-9
        P2.append(pressure)  # Append pressure to P1
        
        sigma_temperature = float(Tesd.get())
        var_VT = np.array([[sigma_volume**2.0, 0.0], [0.0, sigma_temperature**2.0]])
        
        pressure_phase, var_PVT = phase.pressure(volume, temperature_phase, var_VT)
        sigma_PVT = np.sqrt(np.diag(var_PVT))
        
        P_esd2.append(sigma_PVT[0] * 1.0e-9)  # Append error to P_esd1
        
    return T, P2, P_esd2

def save_pressures_to_file(T, P2, P_esd2, filename):
    # Read the directory path from path.txt
    with open('path.txt', 'r') as file:
        file_path = file.readline().strip()  # Get the directory path
    
    # Extract the directory path
    dir_path = os.path.dirname(file_path)
    
    # Ensure the directory exists
    os.makedirs(dir_path, exist_ok=True)
    
    # Construct the full output file path
    filepath = os.path.join(dir_path, filename)
    
    # Write the pressures and errors to the file
    with open(filepath, 'w') as file:
        file.write("Temperature (K)\tPressure (GPa)\tError (GPa)\n")
        for t, p, p_esd in zip(T, P2, P_esd2):
            file.write(f"{t:.2f}\t{p:.6f}\t{p_esd:.6f}\n")

def EoS2(a, a0, T, label_P, pressure, temperature):
    global P_EoS2
    
    phase = Mo()
    Na = 6.0221367e+23
    volume = (Na / phase.params['Z']) * (float(a.get()) ** 3) * (10 ** -30)
    phase.params['V_0'] = (Na / phase.params['Z']) * (float(a0.get()) ** 3) * (10 ** -30)
    T = float(T.get())
    
    P = phase.pressure_function(volume, T, phase.params) * 1.0e-9
    P_EoS2 = P
    label_P.config(text=" %.3f" % P)
    
    temperature = np.linspace(10., 3000., 500)
    pressure = np.empty_like(temperature)
    
    for i in range(len(pressure)):
        pressure[i] = phase.pressure_function(volume, temperature[i], phase.params)
    
    with open('P_2.txt', 'w') as f:
        f.write(','.join(map(str, pressure)))
    
    with open('T_2.txt', 'w') as f:
        f.write(','.join(map(str, temperature)))
        
    return
    
def EoS2high(a, ahigh, a0, T, label_P, pressure, temperature, Tesd):
    global P_esd2
    
    phase = Mo()
    Na = 6.0221367e+23
    volume = (Na / phase.params['Z']) * (float(a.get()) + float(ahigh.get())) ** 3 * (10 ** -30)
    phase.params['V_0'] = (Na / phase.params['Z']) * (float(a0.get()) ** 3) * (10 ** -30)
    T = float(T.get())
   
    P = phase.pressure_function(volume, T, phase.params) * 1.0e-9
    #P_esd2 = np.abs(P_EoS2 - P)
    label_P.config(text=" %.3f" % P)
    
    temperature = np.linspace(10., 3000., 500)
    pressure = np.empty_like(temperature)
    
    for i in range(len(pressure)):
        pressure[i] = phase.pressure_function(volume, temperature[i], phase.params)
    
    with open('P_2h.txt', 'w') as f:
        f.write(','.join(map(str, pressure)))
    
    with open('T_2h.txt', 'w') as f:
        f.write(','.join(map(str, temperature)))

    # Code segment below adapted from Burnman file 'examples_calibrants.py'
    volume_phase = ((Na / phase.params['Z']) * (float(a.get()) ** 3) * (10 ** -30))  #m3/mol
    s_volume = 3 * ((float(a.get())) ** 2) * float(ahigh.get())
    sigma_volume = ((Na / phase.params['Z']) * (s_volume ** 3) * (10 ** -30)) 
    
    temperature_phase = T
    sigma_temperature = float(Tesd.get())
    
    print("Experimental observations:")
    print(f"Volume: {volume_phase*1.e6:.4f} +/- {sigma_volume*1.e6:.4f} m^3/mol")
    print(f"Temperature: {temperature_phase} +/- {sigma_temperature} K")
    print()
    
    # Output a covariance matrix
    var_VT = np.array([[sigma_volume**2.0, 0.0], [0.0, sigma_temperature**2.0]])

    pressure, var_PVT = phase.pressure(volume_phase, temperature_phase, var_VT)
    
    # Standard conversion from the covariance matrix to the
    # standard deviations and correlation matrix
    sigma_PVT = np.sqrt(np.diag(var_PVT))
    corr_PVT = np.einsum("i, ij, j -> ij", 1.0 / sigma_PVT, var_PVT, 1.0 / sigma_PVT)

    P_esd2 = sigma_PVT[0]/1.e9 #GPa

    print(
        "Pressure with propagated uncertainty\n"
        "estimated using the equation of state:"
    )
    print(f"{pressure/1.e9:.4f} +/- {sigma_PVT[0]/1.e9:.4f} GPa")
    print("PVT correlation matrix:")
    print(f"{corr_PVT}")
    print("The uncertainties and correlation matrix only take into account ")
    print("the uncertainties in the measured volume and temperature, ")
    print("not the uncertainties in the calibrated equation of state.")
    print()    
        
    return 

def EoS2low(a, alow, a0, T, label_P, pressure, temperature):
    
    phase = Mo()
    Na = 6.0221367e+23
    volume = (Na / phase.params['Z']) * (float(a.get()) - float(alow.get())) ** 3 * (10 ** -30)
    phase.params['V_0'] = (Na / phase.params['Z']) * (float(a0.get()) ** 3) * (10 ** -30)
    T = float(T.get())
 
    P = phase.pressure_function(volume, T, phase.params) * 1.0e-9
    label_P.config(text=" %.3f" % P)
    
    temperature = np.linspace(10., 3000., 500)
    pressure = np.empty_like(temperature)
    
    for i in range(len(pressure)):
        pressure[i] = phase.pressure_function(volume, temperature[i], phase.params)  
    
    with open('P_2l.txt', 'w') as f:
        f.write(','.join(map(str, pressure)))
    
    with open('T_2l.txt', 'w') as f:
        f.write(','.join(map(str, temperature)))
        
    return     